from .core import Variable, Constant
from .functional import *
from .initializers import *
from .objectives import MeanSquaredError, MeanAbsoluteError, SparseCrossEntropy, CrossEntropy, BinaryCrossEntropy, get_objective

name = 'xshinnosuke-nn'
